"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var hornet_js_utils_1 = require("hornet-js-utils");
var logger_1 = require("hornet-js-logger/src/logger");
var CLS_KEY = "hornet.timers";
var Timer = /** @class */ (function () {
    function Timer() {
    }
    /**
     * Permet de définir un autre CLS que celui par défaut
     * @param cls
     */
    Timer.setClsConfiguration = function (cls) {
        if (typeof window === "undefined") {
            global._TimerUseCLS = cls;
        }
        else {
            window._TimerUseCLS = cls;
        }
    };
    /**
     * Récupération de la configuration du CLS
     * @return cls
     */
    Timer.getClsConfiguration = function () {
        if (typeof window === "undefined") {
            return global._TimerUseCLS;
        }
        else {
            return window._TimerUseCLS;
        }
    };
    /**
     * Démarre un timer et en arrête un autre s'il est renseigné.
     * @param {string} timerName id du timer à démarrer
     * @param {string}  stopTimerName id du timer à arrêter
     */
    Timer.startTimer = function (timerName, stopTimerName) {
        var timers = Timer.getTimers();
        if (!timers) {
            timers = {};
        }
        if (!timers[timerName]) {
            timers[timerName] = [];
        }
        if (timers[timerName].length % 2 !== 0) {
            Timer.logger.debug("startTimer : Timers problem (" + timerName + ") not stopped.");
        }
        else {
            timers[timerName].push(new Date().getTime());
            Timer.setTimers(timers);
        }
        if (stopTimerName) {
            Timer.stopTimer(stopTimerName);
        }
    };
    /**
     * Arrête un timer
     * @param {string} timerName id du timer à arrêter
     */
    Timer.stopTimer = function (timerName) {
        var timers = Timer.getTimers();
        if (!timers || !timers[timerName] || timers[timerName].length % 2 === 0) {
            Timer.logger.debug("startTimer : Timers problem (" + timerName + ") not started.");
        }
        else {
            timers[timerName].push(new Date().getTime());
            Timer.setTimers(timers);
        }
    };
    /**
     * Arrête tous les timers
     */
    Timer.stopAllTimers = function () {
        var timers = Timer.getTimers();
        if (timers) {
            Object.keys(timers).forEach(function (timerName) {
                if (timers[timerName].length % 2 !== 0) {
                    timers[timerName].push(new Date().getTime());
                }
            });
            Timer.setTimers(timers);
        }
    };
    /**
     * Fonction pour logger les timers se basant sur Timer.TIMERS_ORDER et le cls (hornet.timers)
     * @param {...string} _toLog - informations supplémentaires à logger
     */
    Timer.logFinally = function () {
        var _toLog = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            _toLog[_i] = arguments[_i];
        }
        var timers = Timer.getTimers();
        var valueTimers = {};
        if (timers) {
            var log_1 = "Metrology : " + (_toLog && _toLog.map(function (val) { return val === null || typeof val === "undefined" ? "<unknow value>" : val; }).join(" - "));
            var formatLogTimer = function (timer) {
                if (timers[timer]) {
                    if (timers[timer].length % 2 !== 0) {
                        Timer.logger.debug("Timers problem (" + timer + ") not stopped. Add automatic stop.");
                        timers[timer].push(new Date().getTime());
                    }
                    for (var tCount = 0; tCount < timers[timer].length; tCount += 2) {
                        var tValue = timers[timer][tCount + 1] - timers[timer][tCount];
                        valueTimers[timer] = valueTimers[timer] ? valueTimers[timer] + tValue : tValue;
                    }
                    log_1 += " - " + timer + " " + valueTimers[timer] + " ms";
                }
            };
            // Constitution du message suivant l'ordre des Timers
            Timer.TIMERS_ORDER.forEach(formatLogTimer);
            // Ajout des Timers non contenu dans TIMERS_ORDER
            Object.keys(timers).filter(function (timerKey) { return this.indexOf(timerKey) === -1; }, Timer.TIMERS_ORDER).sort().forEach(formatLogTimer);
            Timer.logger.info(log_1);
        }
        else {
            Timer.logger.warn("No timers available.");
        }
    };
    /**
     * Récupération des Timers enregistrés
     * @param {string} [clsKey=hornet.timers] - clé de cls contenant les timers.
     * @return {object|undefined}
     */
    Timer.getTimers = function (clsKey) {
        if (clsKey === void 0) { clsKey = CLS_KEY; }
        var timers;
        try {
            var ns = Timer.getClsConfiguration();
            if (ns === true || ns === null || typeof ns === "undefined") {
                timers = hornet_js_utils_1.Utils.getCls(clsKey);
            }
            else if (ns === false) {
                timers = Timer._timers;
            }
            else if (ns) {
                timers = ns.get(clsKey);
            }
            else {
                throw new Error("Timer : UseCLS value undefined.");
            }
        }
        catch (e) {
            Timer.logger.warn("No timers available, Cls error.");
            Timer.logger.warn("error : ", e);
        }
        return timers;
    };
    /**
     * Enregistrements des Timers (écrasement complet)
     * @param {object} timers - clé de cls contenant les timers.
     * @param {string} [clsKey=hornet.timers] - clé de cls contenant les timers.
     */
    Timer.setTimers = function (timers, clsKey) {
        if (clsKey === void 0) { clsKey = CLS_KEY; }
        try {
            var ns = Timer.getClsConfiguration();
            if (ns === true || ns === null || typeof ns === "undefined") {
                hornet_js_utils_1.Utils.setCls(clsKey, timers);
            }
            else if (ns === false) {
                Timer._timers = timers;
            }
            else if (ns) {
                ns.set(clsKey, timers);
            }
            else {
                throw new Error("Timer : UseCLS value undefined.");
            }
        }
        catch (e) {
            Timer.logger.warn("No timers available, Cls error.");
            Timer.logger.warn("error : ", e);
        }
    };
    Timer.TIMER_REQUEST = "REQUEST";
    Timer.TIMER_ACTION = "ACTION";
    Timer.TIMER_SERVICE = "SERVICE";
    Timer.TIMER_AJAX_REQUEST = "REQUEST_AJAX";
    Timer.TIMER_SERVER_HTTP = "HTTP_ROUTER";
    /**
     * Ordre des timers activés par défaut
     */
    Timer.TIMERS_ORDER = [Timer.TIMER_REQUEST, Timer.TIMER_SERVER_HTTP, Timer.TIMER_ACTION, Timer.TIMER_SERVICE, Timer.TIMER_AJAX_REQUEST];
    Timer.NS_PER_SEC = 1e9;
    Timer.NS_TO_MS_PER_SEC = 1e6;
    Timer.logger = logger_1.Logger ? logger_1.Logger.getLogger("hornet-js-core.timers.Timer") : console;
    return Timer;
}());
exports.Timer = Timer;

//# sourceMappingURL=timer.js.map
